;;; bench.el --- benchmark read-extended-command -*- lexical-binding: t -*-

;; Load libraries with command-modes and completion-predicates.

(require 'bookmark)
(require 'dbus)
(require 'eww)
(require 'finder)
(require 'gnus-salt)
(require 'info)
(require 'man)
(require 'shr)
(require 'tramp-cmds)
(require 'tramp-crypt)

;; Custom read-extended-command-predicates.

(defun bench--safe (symbol buffer)
  (if (get symbol 'completion-predicate)
      (condition-case-unless-debug err
          (funcall (get symbol 'completion-predicate) symbol buffer)
        (error
         (message "command-completion-default-include-p: %s: %S" symbol err)
         nil))
    (or (null (command-modes symbol))
        (command-completion-using-modes-p symbol buffer))))

(defun bench--expensive (_symbol _buffer)
  (let ((i 1000))
    (dotimes (_ i)
      (setq i (1+ i)))
    i))

;; Benchmarks.

(defun bench-pred-expensive ()
  (let ((read-extended-command-predicate #'bench--expensive))
    (read-extended-command)))

(defun bench-pred-none ()
  (let ((read-extended-command-predicate nil))
    (read-extended-command)))

(defun bench-pred-old ()
  (let ((read-extended-command-predicate
         #'command-completion-default-include-p))
    (read-extended-command)))

(defun bench-pred-new ()
  (let ((read-extended-command-predicate #'bench--safe))
    (read-extended-command)))

(with-current-buffer-window "*bench*" nil nil
  (pcase-dolist (`(,reps . ,keys)
                 '((10 . "\C-i\C-m")
                   (100 . "eww-\C-i\C-a\C-k\C-m")))
    (insert (if (bobp) "" ?\n) (key-description keys))
    (setq keys (listify-key-sequence keys))
    (dotimes (_ 3)
      (insert ?\n)
      (dolist (fn '(bench-pred-expensive
                    bench-pred-none
                    bench-pred-old
                    bench-pred-new))
        (dotimes (_ 3)
          (garbage-collect)
          (insert (apply #'format "%-20s %f %d %f\n" fn
                         (benchmark-run reps
                           (setq unread-command-events keys)
                           (funcall fn)))))))))

;;; bench.el ends here
